package com.digertech.sdk.samples;

import java.io.File;
import java.io.RandomAccessFile;
import java.util.Vector;

import com.digertech.sdk.reader.*;
import com.digertech.sdk.records.*;
import com.digertech.sdk.records.x9100.*;
import com.digertech.sdk.records.x937.*;
import com.digertech.sdk.writer.DTFileWriter;

/**
* This sample writes ICL records to an ICL file.
* 
* To run this sample, a jar file in the lib folder should 
* be included in the classpath:
*   FileProcessingSDK.jar
*   
* Execute the commmand to run the sample :
*  java com.digertech.sdk.samples.SampleICLFileWriter [ICL file] [license key]
*           [ICL file]    - ICL file full path name.
*           [license key] - File processing SDK license key.
*/
public class SampleICLFileWriter {
	
	private long bundleAmount     = 0;
	private long cashLeterAmount  = 0;
	private long fileAmount       = 0;
	
	private int  bundleItem       = 0;
	private int  cashLeterItem    = 0;
	private int  fileItem         = 0;

	private int  bundleImage      = 0;
	private int  cashLeterImage   = 0;

	private int  bundleCount      = 0;
	private int  cashLetterCount  = 0;
	private int  recordCount      = 0;
	
	private boolean forward       = true;
	
	public SampleICLFileWriter() {
		super();
	}
	
	public void writeRecords(File file, int format, boolean forward) throws FileIOException {
		DTFileWriter writer = null;
		byte[] imageData    = null;
		int    imageSize    = 0;
		
		this.forward = forward;
		
		try {
			writer = new DTFileWriter(file);
			
			// all records should be written to the file in order
			// the SDK will not validate the order of the records.
			
			writer.writeRecord(getType01Record(format));
			System.out.println("Write Type 01 record.");
			
				writer.writeRecord(getType10Record(format));
				System.out.println("Write Type 10 record.");
					writer.writeRecord(getType61Record(format));
					System.out.println("Write Type 61 record.");
//					writer.writeRecord(getType61V2Record(format));
//					System.out.println("Write Type 61 V2 record.");
//					writer.writeRecord(getType61V3Record(format));
//					System.out.println("Write Type 61 V3 record.");
//					writer.writeRecord(getType61V4Record(format));
//					System.out.println("Write Type 61 V4 record.");
//					writer.writeRecord(getType61V5Record(format));
//					System.out.println("Write Type 61 V5 record.");

					writer.writeRecord(getType20Record(format));
					System.out.println("Write Type 20 record.");
					if (forward) {
						writer.writeRecord(getType25Record(format));
						System.out.println("Write Type 25 record.");
						
							// add user record to the file
							writer.writeRecord(getUserRecord(format));
							System.out.println("Write user record.");
							
							writer.writeRecord(getType26Record(format));
							System.out.println("Write Type 26 record.");
							writer.writeRecord(getType27Record(format));
							System.out.println("Write Type 27 record.");
							writer.writeRecord(getType28Record(format));
							System.out.println("Write Type 28 record.");
					} else {
						writer.writeRecord(getType31Record(format));
						System.out.println("Write Type 31 record.");
							writer.writeRecord(getType32Record(format));
							System.out.println("Write Type 32 record.");
							writer.writeRecord(getType33Record(format));
							System.out.println("Write Type 33 record.");
							writer.writeRecord(getType34Record(format));
							System.out.println("Write Type 34 record.");
							writer.writeRecord(getType35Record(format));
							System.out.println("Write Type 35 record.");
					}
							imageData = getImageData(true);
							imageSize = imageData == null ? 0 : imageData.length;
							writer.writeRecord(getType50Record(format, imageSize, true));
							System.out.println("Write Type 50 record.");
							writer.writeRecord(getType52Record(format, imageData));
							System.out.println("Write Type 52 record.");
							writer.writeRecord(getType54Record(format));
							System.out.println("Write Type 54 record.");

							imageData = getImageData(false);
							imageSize = imageData == null ? 0 : imageData.length;
							writer.writeRecord(getType50Record(format, imageSize, false));
							System.out.println("Write Type 50 record.");
							writer.writeRecord(getType52Record(format, imageData));
							System.out.println("Write Type 52 record.");
							writer.writeRecord(getType54Record(format));
							System.out.println("Write Type 54 record.");
					writer.writeRecord(getType70Record(format));
					System.out.println("Write Type 70 record.");
				writer.writeRecord(getType90Record(format));
				System.out.println("Write Type 90 record.");
			writer.writeRecord(getType99Record(format));
			System.out.println("Write Type 99 record.");
		} catch (RecordException ex) {
			throw new FileIOException(ex.toString());
		} catch (Exception ex) {
			throw new FileIOException(ex.toString());
		} finally {
			if (writer != null) {
				writer.close();
				System.out.println("File is closed.");
			}
		}
	}
	
	private byte[] getImageData(boolean front) {
		byte[] imageData = null;
		RandomAccessFile raf = null;

		try {
			if (front) {
				raf = new RandomAccessFile("images/front.tif", "r");
			} else {
				raf = new RandomAccessFile("images/back.tif", "r");
			}
			imageData = new byte[(int)raf.length()];
			raf.read(imageData);
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			if (raf != null) {
				try {
					raf.close();
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}
		
		return imageData; 
	}
	
	private X9Record getType01Record(int format) throws RecordException {
		this.fileAmount = 0;
		this.fileItem = 0;
		this.cashLetterCount = 0;
		this.recordCount++;

		if (format == RecordDefines.X9100_180) {
			X9100Type01Record t01 = new X9100Type01Record(RecordDefines.X9100_180);

			t01.setStandardLevel("20"); 
			t01.setTestFileInd("P");  
			t01.setDestRoutingNum("123456789");    
			t01.setOriginRoutingNum("987654321"); 
			t01.setFileCreateDate("20090805");  
			t01.setFileCreateTime("0828"); 
			t01.setResendInd("N");     
			t01.setDestName("Destation Bank");      
			t01.setOriginName("Origin Bank");      
			t01.setFileIdModifier(""); 
			t01.setCountryCode("");  
			t01.setUserField("");      
			t01.setReserved("");     
			
			return t01;
		} else {
			X937Type01Record t01 = new X937Type01Record(RecordDefines.X937_2003);
			
			t01.setStandardLevel("03"); 
			t01.setTestFileInd("P");  
			t01.setDestRoutingNum("123456789");    
			t01.setOriginRoutingNum("987654321"); 
			t01.setFileCreateDate("20090805");  
			t01.setFileCreateTime("0828"); 
			t01.setResendInd("N");     
			t01.setDestName("Destation Bank");      
			t01.setOriginName("Origin Bank");      
			t01.setFileIdModifier(""); 
			t01.setCountryCode("");  
			t01.setUserField("");      
			t01.setReserved("");     
			
			return t01;
		}
	}
	
	private X9Record getType10Record(int format) throws RecordException {
		
		this.cashLeterAmount = 0;
		this.cashLeterImage = 0;
		this.cashLeterItem = 0;
		this.bundleCount = 0;
		this.cashLetterCount ++;
		this.recordCount++;
		
		if (format == RecordDefines.X9100_180) {
			X9100Type10Record t10 = new X9100Type10Record(RecordDefines.X9100_180);
			
			t10.setCollectionTypeInd(this.forward ? "01" : "03");
			t10.setDestRoutingNum("123456789");
			t10.setInstitutionRoutingNum("987654321");
			t10.setBusinessDate("20090805");
			t10.setCreateDate("20090805");
			t10.setCreateTime("0828");
			t10.setRecordTypeInd("F");
			t10.setDocTypeInd("C");
			t10.setCashletterID("21153223");
			t10.setOrigContactName("");
			t10.setOrigContactPhone("");
			t10.setFedWorkType("");
			t10.setUserField("");
			
			return t10;
		} else {
			X937Type10Record t10 = new X937Type10Record(RecordDefines.X937_2003);
			
			t10.setCollectionTypeInd("01");
			t10.setDestRoutingNum("123456789");
			t10.setInstitutionRoutingNum("987654321");
			t10.setBusinessDate("20090805");
			t10.setCreateDate("20090805");
			t10.setCreateTime("0828");
			t10.setRecordTypeInd("F");
			t10.setDocTypeInd("C");
			t10.setCashletterID("21153223");
			t10.setOrigContactName("");
			t10.setOrigContactPhone("");
			t10.setFedWorkType("");
			t10.setUserField("");
			t10.setReserved("");
			
			return t10;
		}
	}

	private X9Record getType20Record(int format) throws RecordException {
	
		this.bundleAmount = 0;
		this.bundleImage = 0;
		this.bundleItem = 0;
		this.bundleCount++;
		this.recordCount++;
		
		if (format == RecordDefines.X9100_180) {
			X9100Type20Record t20 = new X9100Type20Record(RecordDefines.X9100_180);
			
			t20.setCollectionTypeInd(this.forward ? "01" : "03");
			t20.setDestRoutingNum("123456789");
			t20.setInstitutionRoutingNum("987654321");
			t20.setBusinessDate("20090805");
			t20.setCreateDate("20090805");
			t20.setBundleID("0000100400");
			t20.setBundleSeqNum("0001");
			t20.setCycleNum("02");
			t20.setReturnRoutingNum("100000010");
			t20.setBundleCreateTime("0829");
			t20.setUserField("");
			t20.setReserved("");
			
			return t20;
		} else {
			X937Type20Record t20 = new X937Type20Record(RecordDefines.X937_2003);
			
			t20.setCollectionTypeInd("01");
			t20.setDestRoutingNum("123456789");
			t20.setInstitutionRoutingNum("987654321");
			t20.setBusinessDate("20090805");
			t20.setCreateDate("20090805");
			t20.setBundleID("0000100400");
			t20.setBundleSeqNum("0001");
			t20.setCycleNum("02");
			t20.setReturnRoutingNum("100000010");
			t20.setUserField("");
			t20.setReserved("");
			
			return t20;
		}
	}

	private X9Record getType25Record(int format) throws RecordException {

		this.recordCount++;
		this.bundleAmount += 19800;
		this.cashLeterAmount += 19800;
		this.fileAmount += 19800;
		this.bundleItem++;
		this.cashLeterItem++;
		this.fileItem++;
		
		if (format == RecordDefines.X9100_180) {
			X9100Type25Record t25 = new X9100Type25Record(RecordDefines.X9100_180);

			t25.setAuxOnUs("         100008");
			t25.setExtProcCode("");
			t25.setRoutingNum("123456789");
			t25.setOnUsField("        98989898618/");
			t25.setAmount(19800);
			t25.setItemSeqNum("000002800012001");
			t25.setDocTypeInd("C");
			t25.setRetAcceptInd("6");
			t25.setMicrValidInd("");
			t25.setBofdInd("U");
			t25.setAddendumCount(3);
			t25.setCorrectionInd("");
			t25.setArchiveTypeInd("");
			
			return t25;
		} else {
			X937Type25Record t25 = new X937Type25Record(RecordDefines.X937_2003);

			t25.setAuxOnUs("         100008");
			t25.setExtProcCode("");
			t25.setRoutingNum("12345678");
			t25.setRoutingNumCheckDigit("9");
			t25.setOnUsField("        98989898618/");
			t25.setAmount(19800);
			t25.setItemSeqNum("000002800012001");
			t25.setDocTypeInd("C");
			t25.setRetAcceptInd("6");
			t25.setMicrValidInd("");
			t25.setBofdInd("U");
			t25.setAddendumCount(3);
			t25.setCorrectionInd("");
			t25.setArchiveTypeInd("");
			
			return t25;
		}
	}

	private X9Record getType26Record(int format) throws RecordException {

		this.recordCount++;

		if (format == RecordDefines.X9100_180) {
			X9100Type26Record t26 = new X9100Type26Record(RecordDefines.X9100_180);

			t26.setAddendumNum("1");
			t26.setBofdRoutingNum("987654321");
			t26.setBofdBusinessDate("20090805");
			t26.setBofdItemSeqNum("000002800012001");
			t26.setBofdDepositAccountNum("");
			t26.setBofdDepositBranch("");
			t26.setPayeeName("");
			t26.setTruncationInd("Y");
			t26.setBofdConversionInd("2");
			t26.setBofdCorrectionInd("0");
			t26.setMagneticReadInd("3");
			t26.setUserField("");
			t26.setReserved("");
			
			return t26;
		} else {
			X937Type26Record t26 = new X937Type26Record(RecordDefines.X937_2003);

			t26.setAddendumNum("1");
			t26.setBofdRoutingNum("987654321");
			t26.setBofdBusinessDate("20090805");
			t26.setBofdItemSeqNum("000002800012001");
			t26.setBofdDepositAccountNum("");
			t26.setBofdDepositBranch("");
			t26.setPayeeName("");
			t26.setTruncationInd("Y");
			t26.setBofdConversionInd("2");
			t26.setBofdCorrectionInd("0");
			t26.setUserField("");
			t26.setReserved("");
			
			return t26;
		}
	}

	private X9Record getType27Record(int format) throws RecordException {

		this.recordCount++;

		if (format == RecordDefines.X9100_180) {
			X9100Type27Record t27 = new X9100Type27Record(RecordDefines.X9100_180);

			t27.setDescription("Description");
			t27.setImgCaptureDate("20090805");
			t27.setImgCaptureTime("0806");
			t27.setMicroArchiveSeqNum("0123456");
			
			t27.setImgArchiveLocatorLen(34);
			t27.setCapturedUnparsedMICRDataLen(27);
			t27.setCapturedHighUnparsedMICRDataLen(28);
			t27.setCorrectedUnparsedMICRDataLen(29);
			t27.setCorrectedHighUnparsedMICRDataLen(30);
			t27.setUserDataLen(31);
			
			t27.setReserved("");
			
			t27.setImgArchiveLocator("Image Archive Locator");
			t27.setCapturedUnparsedMICRData("capturedUnparsedMICRData");
			t27.setCapturedHighUnparsedMICRData("capturedHighUnparsedMICRData");
			t27.setCorrectedUnparsedMICRData("correctedUnparsedMICRData");
			t27.setCorrectedHighUnparsedMICRData("correctedHighUnparsedMICRData");
			t27.setUserField("UserField");
			
			return t27;
		} else {
			X937Type27Record t27 = new X937Type27Record(RecordDefines.X937_2003);

			t27.setVarRecordInd("0");
			t27.setMicroArchiveSeqNum("0123456");
			t27.setImgArchiveLocatorLen(34);
			t27.setImgArchiveLocator("Image Archive Locator");
			t27.setDescription("Description");
			t27.setUserField("UserField");
			t27.setReserved("");
			
			return t27;
		}
	}

	private X9Record getType28Record(int format) throws RecordException {

		this.recordCount++;

		if (format == RecordDefines.X9100_180) {
			X9100Type28Record t28 = new X9100Type28Record(RecordDefines.X9100_180);

			t28.setAddendumRecordNum("01");
			t28.setRoutingNum("987654321");
			t28.setEndorsingDate("20090805");
			t28.setItemSeqNum("000002800012001");
			t28.setTruncationInd("N");
			t28.setConversionInd("2");
			t28.setCorrectionInd("0");
			t28.setReturnReason("");
			t28.setMagneticReadInd("3");
			t28.setBankId("0");
			t28.setDepositAccount("0123456789");
			t28.setUserField("");
			t28.setReserved("");
			
			return t28;
		} else {
			X937Type28Record t28 = new X937Type28Record(RecordDefines.X937_2003);

			t28.setAddendumRecordNum("01");
			t28.setRoutingNum("987654321");
			t28.setEndorsingDate("20090805");
			t28.setItemSeqNum("000002800012001");
			t28.setTruncationInd("N");
			t28.setConversionInd("2");
			t28.setCorrectionInd("0");
			t28.setReturnReason("");
			t28.setUserField("");
			t28.setReserved("");
			
			return t28;
		}
	}

	private X9Record getType31Record(int format) throws RecordException {

		this.recordCount++;
		this.bundleAmount += 19800;
		this.cashLeterAmount += 19800;
		this.fileAmount += 19800;
		this.bundleItem++;
		this.cashLeterItem++;
		this.fileItem++;
		
		if (format == RecordDefines.X9100_180) {
			X9100Type31Record t31 = new X9100Type31Record(RecordDefines.X9100_180);

			t31.setAuxOnUs("         100008");
			t31.setExtProcCode("");
			t31.setRoutingNum("123456789");
			t31.setOnUs("        98989898618/");
			t31.setAmount(19800);
			t31.setItemSeqNum("000002800012001");
			t31.setAddendumCount(4);
			t31.setReserved("");
			
			return t31;
		} else {
			X937Type31Record t31 = new X937Type31Record(RecordDefines.X937_2003);

			t31.setRoutingNum("12345678");
			t31.setRoutingNumCheckDigit("9");
			t31.setOnUs("        98989898618/");
			t31.setAmount(19800);
			t31.setReturnReason("W");
			t31.setAddendumCount(4);
			t31.setDocTypeInd("C");
			t31.setForwardBundleDate("20090801");
			t31.setItemSeqNum("000002800012001");
			t31.setExtProcCode("");
			t31.setNotificationInd("2");
			t31.setArchiveTypeInd("");
			t31.setReserved("");
			
			return t31;
		}
	}

	private X9Record getType32Record(int format) throws RecordException {

		this.recordCount++;

		if (format == RecordDefines.X9100_180) {
			X9100Type32Record t32 = new X9100Type32Record(RecordDefines.X9100_180);

			t32.setAddendumNum("1");
			t32.setBofdRoutingNum("987654321");
			t32.setBofdBusinessDate("20090805");
			t32.setBofdItemSeqNum("000002800012001");
			t32.setBofdDepositAccountNum("");
			t32.setBofdDepositBranch("");
			t32.setPayeeName("");
			t32.setTruncationInd("Y");
			t32.setBofdConversionInd("2");
			t32.setBofdCorrectionInd("0");
			t32.setMagneticReadInd("3");
			t32.setUserField("");
			t32.setReserved("");
			
			return t32;
		} else {
			X937Type32Record t32 = new X937Type32Record(RecordDefines.X937_2003);

			t32.setAddendumNum("1");
			t32.setBofdRoutingNum("987654321");
			t32.setBofdBusinessDate("20090805");
			t32.setBofdItemSeqNum("000002800012001");
			t32.setBofdDepositAccountNum("");
			t32.setBofdDepositBranch("");
			t32.setPayeeName("");
			t32.setTruncationInd("Y");
			t32.setBofdConversionInd("2");
			t32.setBofdCorrectionInd("0");
			t32.setUserField("");
			t32.setReserved("");
			
			return t32;
		}
	}

	private X9Record getType33Record(int format) throws RecordException {

		this.recordCount++;
		
		if (format == RecordDefines.X9100_180) {
			X9100Type33Record t33 = new X9100Type33Record(RecordDefines.X9100_180);

			t33.setReturnReason("W");
			t33.setForwardBundleDate("20090801");
			t33.setAdditionalReturnReason("U");
			t33.setNumReturned("0");
			t33.setRetDocTypeInd("C");
			t33.setArchiveTypeInd("");
			t33.setPayorBankName("Payor Bank Name");
			t33.setPayBankItemSeqNum("000002800012001");
			t33.setPayBankBusinessDate("20090805");
			t33.setPayAccountName("Pay Account Name");
			
			return t33;
		} else {
			X937Type33Record t33 = new X937Type33Record(RecordDefines.X937_2003);

			t33.setPayorBankName("Payor Bank Name");
			t33.setAuxOnUs("         100008");
			t33.setPayBankItemSeqNum("000002800012001");
			t33.setPayBankBusinessDate("20090805");
			t33.setPayAccountName("Pay Account Name");
			
			return t33;
		}
	}


	private X9Record getType34Record(int format) throws RecordException {

		this.recordCount++;

		if (format == RecordDefines.X9100_180) {
			X9100Type34Record t34 = new X9100Type34Record(RecordDefines.X9100_180);

			t34.setDescription("Description");
			t34.setImgCaptureDate("20090805");
			t34.setImgCaptureTime("0806");
			t34.setMicroArchiveSeqNum("0123456");
			
			t34.setImgArchiveLocatorLen(34);
			t34.setCapturedUnparsedMICRDataLen(27);
			t34.setCapturedHighUnparsedMICRDataLen(28);
			t34.setCorrectedUnparsedMICRDataLen(29);
			t34.setCorrectedHighUnparsedMICRDataLen(30);
			t34.setUserDataLen(31);
			
			t34.setReserved("");
			
			t34.setImgArchiveLocator("Image Archive Locator");
			t34.setCapturedUnparsedMICRData("capturedUnparsedMICRData");
			t34.setCapturedHighUnparsedMICRData("capturedHighUnparsedMICRData");
			t34.setCorrectedUnparsedMICRData("correctedUnparsedMICRData");
			t34.setCorrectedHighUnparsedMICRData("correctedHighUnparsedMICRData");
			t34.setUserField("UserField");
			
			return t34;
		} else {
			X937Type34Record t34 = new X937Type34Record(RecordDefines.X937_2003);

			t34.setVarRecordInd("0");
			t34.setMicroArchiveSeqNum("0123456");
			t34.setImgArchiveLocatorLen(34);
			t34.setImgArchiveLocator("Image Archive Locator");
			t34.setDescription("Description");
			t34.setUserField("UserField");
			t34.setReserved("");
			
			return t34;
		}
	}

	private X9Record getType35Record(int format) throws RecordException {

		this.recordCount++;

		if (format == RecordDefines.X9100_180) {
			X9100Type35Record t35 = new X9100Type35Record(RecordDefines.X9100_180);

			t35.setAddendumRecordNum("01");
			t35.setRoutingNum("987654321");
			t35.setEndorsingDate("20090805");
			t35.setItemSeqNum("000002800012001");
			t35.setTruncationInd("N");
			t35.setConversionInd("2");
			t35.setCorrectionInd("0");
			t35.setReturnReason("");
			t35.setMagneticReadInd("3");
			t35.setBankId("0");
			t35.setDepositAccount("0123456789");
			t35.setUserField("");
			t35.setReserved("");
			
			return t35;
		} else {
			X937Type35Record t35 = new X937Type35Record(RecordDefines.X937_2003);

			t35.setAddendumRecordNum("01");
			t35.setRoutingNum("987654321");
			t35.setEndorsingDate("20090805");
			t35.setItemSeqNum("000002800012001");
			t35.setTruncationInd("N");
			t35.setConversionInd("2");
			t35.setCorrectionInd("0");
			t35.setReturnReason("");
			t35.setUserField("");
			t35.setReserved("");
			
			return t35;
		}
	}

//	@SuppressWarnings("unused")
//	private X9Record getType40Record(int format) throws RecordException {
//
//		this.recordCount++;
//
//		if (format == RecordDefines.X9100_180) {
//			X9100Type40Record t40 = new X9100Type40Record(RecordDefines.X9100_180);
//
//			t40.setDestRoutingNum("987654321");
//			t40.setLowAccount("0123456789");
//			t40.setHighAccount("9876543210");
//			t40.setTotalItemCount(1);
//			t40.setTotalItemAmount(19800);
//			t40.setUserField("");
//			t40.setReserved("");
//			
//			return t40;
//		} else {
//			X937Type40Record t40 = new X937Type40Record(RecordDefines.X937_2003);
//
//			t40.setDestRoutingNum("987654321");
//			t40.setLowAccount("0123456789");
//			t40.setHighAccount("9876543210");
//			t40.setTotalItemCount(1);
//			t40.setTotalItemAmount(19800);
//			t40.setUserField("");
//			t40.setReserved("");
//			
//			return t40;
//		}
//	}
//
//	@SuppressWarnings("unused")
//	private X9Record getType41Record(int format) throws RecordException {
//
//		this.recordCount++;
//
//		if (format == RecordDefines.X9100_180) {
//			X9100Type41Record t41 = new X9100Type41Record(RecordDefines.X9100_180);
//
//			t41.setDestRoutingNum("987654321");
//			t41.setNonHitInd("O");
//			t41.setTotalItemCount(1);
//			t41.setTotalItemAmount(19800);
//			t41.setUserField("");
//			t41.setReserved("");
//			
//			return t41;
//		} else {
//			X937Type41Record t41 = new X937Type41Record(RecordDefines.X937_2003);
//
//			t41.setDestRoutingNum("987654321");
//			t41.setNonHitInd("O");
//			t41.setTotalItemCount(1);
//			t41.setTotalItemAmount(19800);
//			t41.setUserField("");
//			t41.setReserved("");
//			
//			return t41;
//		}
//	}

	private X9Record getType50Record(int format, long imageSize, boolean front) throws RecordException {

		this.recordCount++;

		if (format == RecordDefines.X9100_180) {
			X9100Type50Record t50 = new X9100Type50Record(RecordDefines.X9100_180);

			t50.setImageInd("1");
			t50.setRoutingNum("987654321");
			t50.setCreateDate("20090805");
			t50.setViewFormatInd("00");
			t50.setViewCompression("00");
			t50.setViewType("0");
			t50.setViewSideInd(front ? "0" : "1");
			t50.setViewDescriptor("00");
			t50.setDigSignInd("0");
			t50.setDigSignHashFuncMethod("");
			t50.setDigSignCryptAlgrMethod("");
			t50.setDsaKeySize("0000000000");
			t50.setDigCertfInd("0");
			t50.setDigCertfFormat("");
			t50.setDigCertfConvMethod("");
			t50.setProtectedDataStart("0000000");
			t50.setProtectedDatatLen("0000000");
			t50.setImgRecreateInd("1");
			t50.setImgTestOverrideInd("0");
			t50.setImgCaptureTime("0806");
			t50.setUserField("");
			t50.setReserved("");
			
			return t50;
		} else {
			X937Type50Record t50 = new X937Type50Record(RecordDefines.X937_2003);

			t50.setImageInd("1");
			t50.setRoutingNum("987654321");
			t50.setCreateDate("20090805");
			t50.setViewFormatInd("00");
			t50.setViewCompression("00");
			t50.setViewDataSize(String.valueOf(imageSize));
			t50.setViewSideInd(front ? "0" : "1");
			t50.setViewDescriptor("00");
			t50.setDigSignInd("0");
			t50.setDigSignMethod("00");
			t50.setSecurityKeySize("00000");
			t50.setProtectedDataStart("0000000");
			t50.setProtectedDatatLen("0000000");
			t50.setImgRecreateInd("1");
			t50.setUserField("");
			t50.setReserved("");
			
			return t50;
		}
	}

	private X9Record getType52Record(int format, byte[] imageData) throws RecordException {
		
		this.recordCount++;
		this.bundleImage++;
		this.cashLeterImage++;

		if (format == RecordDefines.X9100_180) {
			X9100Type52Record t52 = new X9100Type52Record(RecordDefines.X9100_180);

			t52.setRoutingNum("987654321");
			t52.setBusinessDate("20090805");
			t52.setCycleNum("02");
			t52.setItemSeqNum("000002800012001");
			
			t52.setSecurityOrigName("");
			t52.setSecurityAuthName("");
			t52.setSecurityKeyName("");
			
			t52.setClippingOrigin("0");
			t52.setClippingCoordinateH1("");
			t52.setClippingCoordinateH2("");
			t52.setClippingCoordinateV1("");
			t52.setClippingCoordinateV2("");
			
			t52.setAncillaryDataInd("0");
			
			t52.setImageRefKeyLen(12);
			t52.setImageRefKey("012345678901");
			
			t52.setDigSigLen(0);
			t52.setDigSig(null);
			
			t52.setDigCertfIssuerDistNameLen(18);
			t52.setDigCertfIssuerDistName("digCertfIssuerDistName");
			
			t52.setDigCertfSerialNumLen(19);
			t52.setDigCertfSerialNum("digCertfSerialNum");
			
			t52.setDigCertfLen(0);
			t52.setDigCertf(null);
			
			if (imageData == null || imageData.length == 0) {
				t52.setImageDataLen(0);
				t52.setImageData(null);
			} else {
				t52.setImageDataLen(imageData.length);
				t52.setImageData(imageData);
			}
			
			t52.setAncillaryDataLen(0);
			t52.setAncillaryData(null);
			
			return t52;
		} else {
			X937Type52Record t52 = new X937Type52Record(RecordDefines.X937_2003);

			t52.setRoutingNum("987654321");
			t52.setBusinessDate("20090805");
			t52.setCycleNum("02");
			t52.setItemSeqNum("000002800012001");
			t52.setSecurityOrigName("");
			t52.setSecurityAuthName("");
			t52.setSecurityKeyName("");
			t52.setClippingOrigin("0");
			t52.setClippingCoordinateH1("");
			t52.setClippingCoordinateH2("");
			t52.setClippingCoordinateV1("");
			t52.setClippingCoordinateV2("");
			t52.setImageRefKeyLen(12);
			t52.setImageRefKey("012345678901");
			t52.setDigSigLen(0);
			t52.setDigSig(null);
			if (imageData == null || imageData.length == 0) {
				t52.setImageDataLen(0);
				t52.setImageData(null);
			} else {
				t52.setImageDataLen(imageData.length);
				t52.setImageData(imageData);
			}
			
			return t52;
		}
	}

	private X9Record getType54Record(int format) throws RecordException {

		this.recordCount++;

		if (format == RecordDefines.X9100_180) {
			X9100Type54Record t54 = new X9100Type54Record(RecordDefines.X9100_180);

			t54.setGlobalImageQuality("0");
			t54.setGlobalImageUsability("0");
			t54.setImageBankSpecificTest("0");
			t54.setPartialImage("0");
			t54.setExcessImageSkew("0");
			t54.setPiggybackImage("0");
			t54.setTooLight("0");
			t54.setTooDark("0");
			t54.setStreakBands("0");
			t54.setBelowMinImageSize("0");
			t54.setExceedMaxImageSize("0");
			t54.setReserved12_24("");

			t54.setImageEnabledPod("0");
			t54.setSourceDocBad("0");
			t54.setDateUsability("0");
			t54.setPayeeUsability("0");
			t54.setConvenientAmountUsability("0");
			t54.setAmountWordsUsability("0");
			t54.setSignatureUsability("0");
			t54.setPayorNameAddressUsability("0");
			t54.setMicrLineUsability("0");
			t54.setMemoLineUsability("0");
			t54.setPayorBankNameAddressUsability("0");
			t54.setPayeeEndorsementUsability("0");
			t54.setBofdEndorsementUsability("0");
			t54.setTransitEndorsementUsability("0");
			t54.setReserved39_44("");

			t54.setUserField("");
			t54.setReserved("");
			
			return t54;
		} else {
			X937Type54Record t54 = new X937Type54Record(RecordDefines.X937_2003);

			t54.setGlobalImageQuality("0");
			t54.setGlobalImageUsability("0");
			t54.setImageBankSpecificTest("0");
			t54.setPartialImage("0");
			t54.setExcessImageSkew("0");
			t54.setPiggybackImage("0");
			t54.setTooLightOrDark("0");
			t54.setStreakBands("0");
			t54.setBelowMinImageSize("0");
			t54.setExceedMaxImageSize("0");
			t54.setReserved12_24("0000000000000");

			t54.setImageEnabledPod("0");
			t54.setSourceDocBad("0");
			t54.setDateUsability("0");
			t54.setPayeeUsability("0");
			t54.setConvenientAmountUsability("0");
			t54.setAmountWordsUsability("0");
			t54.setSignatureUsability("0");
			t54.setPayorNameAddressUsability("0");
			t54.setMicrLineUsability("0");
			t54.setMemoLineUsability("0");
			t54.setPayorBankNameAddressUsability("0");
			t54.setPayeeEndorsementUsability("0");
			t54.setBofdEndorsementUsability("0");
			t54.setTransitEndorsementUsability("0");
			t54.setReserved39_44("000000");

			t54.setUserField("");
			t54.setReserved("");
			
			return t54;
		}
	}

	private X9Record getType61Record(int format) throws RecordException {

		Type61Record t61 = null;
		if (format == RecordDefines.X9100_180) {
			t61 = new X9100Type61Record(RecordDefines.X9100_180);
		} else {
			t61 = new X937Type61Record(RecordDefines.X937_2003);
		}
		
		t61.setRecordUsageId("1");
		t61.setAuxOnUs("   10*89898618*");
		t61.setExtProcessCode("");
		t61.setPostBankRoutingNum("999999999");
		t61.setPostAccountNum("        98989898618/");
		t61.setItemAmount("19800");
		t61.setItemSeqNum("000002800012001");
		t61.setDocTypeInd("K");
		t61.setTypeAccountCode("3");
		t61.setSourceWorkCode("02");
		t61.setReserved("");
		
		return t61;
	}

//	private X9Record getType61V2Record(int format) throws RecordException {
//
//		Type61V2Record t61 = new Type61V2Record(format);
//
//		t61.setAuxOnUs("   10*89898618*");
//		t61.setExtProcessCode("");
//		t61.setPostBankRoutingNum("999999999");
//		t61.setPostAccountNum("        98989898618/");
//		t61.setItemAmount("19800");
//		t61.setItemSeqNum("000002800012001");
//		t61.setDocTypeInd("K");
//		t61.setTypeAccountCode("3");
//		t61.setSourceWorkCode("02");
//		t61.setWorkType("0");
//		t61.setReserved("");
//		
//		return t61;
//	}
//
//	private X9Record getType61V3Record(int format) throws RecordException {
//
//		Type61V3Record t61 = new Type61V3Record(format);
//
//		t61.setRecordUsageId("1");
//		t61.setAuxOnUs("   10*89898618*");
//		t61.setExtProcessCode("");
//		t61.setPostBankRoutingNum("999999999");
//		t61.setPostAccountNum("        98989898618/");
//		t61.setItemAmount("19800");
//		t61.setItemSeqNum("000002800012001");
//		t61.setDocTypeInd("K");
//		t61.setTypeAccountCode("3");
//		t61.setSourceWorkCode("02");
//		t61.setReserved("");
//		
//		return t61;
//	}
//
//	private X9Record getType61V4Record(int format) throws RecordException {
//
//		Type61V4Record t61 = new Type61V4Record(format);
//
//		t61.setAuxOnUs("   10*89898618*");
//		t61.setExtProcessCode("");
//		t61.setPostBankRoutingNum("999999999");
//		t61.setCredit("        98989898618/");
//		t61.setItemAmount("19800");
//		t61.setItemSeqNum("000002800012001");
//		t61.setDocTypeInd("K");
//		t61.setTypeAccountCode("3");
//		t61.setSourceWorkCode("02");
//		t61.setWorkType("0");
//		t61.setDebitCreditInd("0");
//		t61.setReserved("");
//		
//		return t61;
//	}
//
//	private X9Record getType61V5Record(int format) throws RecordException {
//
//		Type61V5Record t61 = new Type61V5Record(format);
//
//		t61.setCreditAmount("000089898618");
//		t61.setCreditAccountNum("     10*89898618*");
//		t61.setProcessingControl("999999");
//		t61.setRoutingNum("888888888");
//		t61.setSerialNum("   98989898618/");
//		t61.setItemSeqNum("000002800012001");
//		t61.setExtProcessCode("");
//		t61.setTypeAccountCode("3");
//		t61.setSourceWorkCode("02");
//		t61.setReserved("");
//		
//		return t61;
//	}

	private X9Record getType70Record(int format) throws RecordException {

		this.recordCount++;

		if (format == RecordDefines.X9100_180) {
			X9100Type70Record t70 = new X9100Type70Record(RecordDefines.X9100_180);

			t70.setItemCount(this.bundleItem);
			t70.setTotalAmount(this.bundleAmount);
			t70.setMicrTotalAmount(0);
			t70.setImageCount(this.bundleImage);
			t70.setCreditTotalAmount(0);
			t70.setCreditItemCount(0);
			t70.setUserField("");
			t70.setReserved("");
			
			return t70;
		} else {
			X937Type70Record t70 = new X937Type70Record(RecordDefines.X937_2003);

			t70.setItemCount(this.bundleItem);
			t70.setTotalAmount(this.bundleAmount);
			t70.setMicrTotalAmount(0);
			t70.setImageCount(this.bundleImage);
			t70.setUserField("");
			t70.setReserved("");
			
			return t70;
		}
	}

//	@SuppressWarnings("unused")
//	private X9Record getType75Record(int format) throws RecordException {
//
//		this.recordCount++;
//
//		if (format == RecordDefines.X9100_180) {
//			X9100Type75Record t75 = new X9100Type75Record(RecordDefines.X9100_180);
//
//			t75.setDestRoutingNum("987654321");
//			t75.setSeqNum("001");
//			t75.setBundleCount(1);
//			t75.setNumberId("00000001");
//			t75.setTotalAmount(this.bundleAmount);
//			t75.setTotalCreditAmount(0);
//			t75.setReserved("");
//			
//			return t75;
//		} else {
//			X937Type75Record t75 = new X937Type75Record(RecordDefines.X937_2003);
//
//			t75.setDestRoutingNum("987654321");
//			t75.setSeqNum("001");
//			t75.setBundleCount(1);
//			t75.setNumberId("00000001");
//			t75.setTotalAmount(this.bundleAmount);
//			t75.setReserved("");
//			
//			return t75;
//		}
//	}
//
//	@SuppressWarnings("unused")
//	private X9Record getType85Record(int format) throws RecordException {
//
//		this.recordCount++;
//
//		if (format == RecordDefines.X9100_180) {
//			X9100Type85Record t85 = new X9100Type85Record(RecordDefines.X9100_180);
//
//			t85.setRoutingNum("987654321");
//			t85.setTotalAmount(this.bundleAmount);
//			t85.setItemCount(this.bundleItem);
//			t85.setCreditTotalAmount(0);
//			t85.setUserField("");
//			t85.setReserved("");
//			
//			return t85;
//		} else {
//			X937Type85Record t85 = new X937Type85Record(RecordDefines.X937_2003);
//
//			t85.setRoutingNum("987654321");
//			t85.setTotalAmount(this.bundleAmount);
//			t85.setItemCount(this.bundleItem);
//			t85.setUserField("");
//			t85.setReserved("");
//			
//			return t85;
//		}
//	}

	private X9Record getType90Record(int format) throws RecordException {

		this.recordCount++;

		if (format == RecordDefines.X9100_180) {
			X9100Type90Record t90 = new X9100Type90Record(RecordDefines.X9100_180);
		
			t90.setBundleCount(this.bundleCount);
			t90.setItemCount(this.cashLeterItem);
			t90.setTotalAmount(this.cashLeterAmount);
			t90.setImageCount(this.cashLeterImage);
			t90.setInstitutionName("");
			t90.setSettlementDate("");
			t90.setCreditItemCount(0);
			t90.setCreditTotalAmount(0);
			
			return t90;
		} else {
			X937Type90Record t90 = new X937Type90Record(RecordDefines.X937_2003);
		
			t90.setBundleCount(this.bundleCount);
			t90.setItemCount(this.cashLeterItem);
			t90.setTotalAmount(this.cashLeterAmount);
			t90.setImageCount(this.cashLeterImage);
			t90.setInstitutionName("");
			t90.setSettlementDate("");
			t90.setReserved("");
			
			return t90;
		}
	}

	private X9Record getType99Record(int format) throws RecordException {

		this.recordCount++;

		if (format == RecordDefines.X9100_180) {
			X9100Type99Record t99 = new X9100Type99Record(RecordDefines.X9100_180);

			t99.setCashletterCount(this.cashLetterCount);
			t99.setTotalRecordCount(this.recordCount);
			t99.setTotalItemCount(this.fileItem);
			t99.setTotalAmount(this.fileAmount);
			t99.setContactName("");
			t99.setContactPhone("");
			t99.setCreditTotalAmount(0);
			
			return t99;
		} else {
			X937Type99Record t99 = new X937Type99Record(RecordDefines.X937_2003);

			t99.setCashletterCount(this.cashLetterCount);
			t99.setTotalRecordCount(this.recordCount);
			t99.setTotalItemCount(this.fileItem);
			t99.setTotalAmount(this.fileAmount);
			t99.setContactName("");
			t99.setContactPhone("");
			t99.setReserved("");
			
			return t99;
		}
	}


	private X9Record getUserRecord(int format) throws RecordException {

		this.recordCount++;

		UserRecord uRec = new UserRecord(format, 61);
		
		uRec.setFieldValue(1, "61");
		uRec.setFieldValue(2, "22210*89898618*");
		uRec.setFieldValue(3, "3");
		uRec.setFieldValue(4, "999999999");
		uRec.setFieldValue(5, "5555555598989898618/");
		uRec.setFieldValue(6, "6666619800");
//		uRec.setFieldValue(7, "000002800012001");
		uRec.setFieldValue(7, leftPad("2800012001", 15, ' '));
		uRec.setFieldValue(8, "K");
		uRec.setFieldValue(9, "3");
		uRec.setFieldValue(10, "2");
		uRec.setFieldValue(11, "0");
		uRec.setFieldValue(12, "0");
		uRec.setFieldValue(13, "RRR");
			
		return uRec;
	}
	
	// len is the length of record field,   c is the pad value like ' '
	public static String leftPad(String s, int len, char c) {
		if (s == null) s = "";
		
	    while (s.length() < len) s = c + s;
	 
	    return s.substring(s.length()-len);
	}
	
	public static void addUserRecord() {
		// Define all your record format here
		
		// Sample: 61 record
		Vector<FieldDefines> t61Defines = new Vector<FieldDefines>();
		FieldDefines field = null;
		  																  // Field,                 field_name,  Position,    Size
		field = new FieldDefines(1, "Record Type",                 1, 2); //     1,                Record Type,   01 C 02,    2  
		t61Defines.add(field);
		field = new FieldDefines(2, "Auxiliary On-Us",             3, 15);//     2,            Auxiliary On-Us,   03 C 17,   15
		t61Defines.add(field);
		field = new FieldDefines(3, "External Processing Code",   18, 1); //     3,   External Processing Code,   18 C 18,    1
		t61Defines.add(field);
		field = new FieldDefines(4, "Payor Bank Routing Number",  19, 9); //     4,  Payor Bank Routing Number,   19 C 27,    9
		t61Defines.add(field);
		field = new FieldDefines(5, "Credit Account Number",      28, 20);//     5,      Credit Account Amount,   28 C 47,   20
		t61Defines.add(field);
		field = new FieldDefines(6, "Item Amount",                48, 10);//     6,                Item Amount,   48 C 57,   10
		t61Defines.add(field);
		field = new FieldDefines(7, "Item Sequence Number",       58, 15);//     7,       Item Sequence Number,   58 C 72,   15
		t61Defines.add(field);
		field = new FieldDefines(8, "Documentation Type Indicator", 73, 1);//    8, Documentation Type Indicator, 73 C 73,    1
		t61Defines.add(field);
		field = new FieldDefines(9, "Type of Account Code",       74, 1); //     9,       Type of Account Code,   74 C 74,    1
		t61Defines.add(field);
		field = new FieldDefines(10, "Source of Work Code",       75, 1); //    10,        Source of Work Code,   75 C 75,    1
		t61Defines.add(field);
		field = new FieldDefines(11, "Work Type",                 76, 1); //    11,                  Work Type,   76 C 76,    1
		t61Defines.add(field); 
		field = new FieldDefines(12, "Debit Credit Indicator",    77, 1); //    12,       Item Sequence Number,   77 C 77,    1
		t61Defines.add(field);
		field = new FieldDefines(13, "Reserved",                  78, 3); //    13,                   Reserved,   78 C 80,    3
		t61Defines.add(field);
		
		DTFileReader.addRecordDefines(61, t61Defines);
	}

//	private X9Record getUserRecord(int format) throws RecordException {
//
//		this.recordCount++;
//
//		UserRecord uRec = new UserRecord(format, 68);
//
//		uRec.setFieldValue(1, "68");
//		uRec.setFieldValue(2, "0");
//		uRec.setFieldValue(3, "888888888");
//		uRec.setFieldValue(4, "99999999999999999999");
//		uRec.setFieldValue(5, "123");
//		uRec.setFieldValue(6, "555");
//		uRec.setFieldValue(7, "0000034");
//		uRec.setFieldValue(8, "                                  ");
//			
//		return uRec;
//	}
//	
//	public static void addUserRecord() {
//		// Define all your record format here
//		
//		// Sample: 68 record
//		Vector<FieldDefines> t68Defines = new Vector<FieldDefines>();
//		FieldDefines field = null;
//		
//		  																  // Field,                 field_name,  Position,    Size
//		field = new FieldDefines(1, "Record Type",                 1, 2); //     1,                Record Type,   01 C 02,    2  
//		t68Defines.add(field);
//		field = new FieldDefines(2, "Owner Identifier Indicator",  3, 1); //     2, Owner Identifier Indicator,   03 C 03,    1
//		t68Defines.add(field);
//		field = new FieldDefines(3, "Owner Identifier",            4, 9); //     3,           Owner Identifier,   04 C 12,    9
//		t68Defines.add(field);
//		field = new FieldDefines(4, "Owner Identifier Modifier",  13, 20);//     4,  Owner Identifier Modifier,   13 C 32,   20
//		t68Defines.add(field);
//		field = new FieldDefines(5, "User Record Format Type",    33, 3); //     5,    User Record Format Type,   33 C 35,    3
//		t68Defines.add(field);
//		field = new FieldDefines(6, "Format Type Version Level",  36, 3); //     6,  Format Type Version Level,   36 C 38,    3
//		t68Defines.add(field);
//		field = new FieldDefines(7, "Length of User Data",        39, 7); //     7,        Length of User Data,   39 C 45,    3
//		t68Defines.add(field);
//		field = new FieldDefines(8, "User Field",                 46, 34); //    8,                 User Field,   46 C 80,   34
//		t68Defines.add(field);
//		
//		DTFileReader.addRecordDefines(68, t68Defines);
//	}

	public static void setLicenseKey() {
		// The license key contains 25 chars. You will recieve the license key after you
		// purchase the SDK product.
		String licenseKey = "XXXXXXXXXXXXXXXXXXXXXXXXX";

		setLicenseKey(licenseKey);
	}
	public static void setLicenseKey(String licenseKey) {
		System.out.println("***********************************************");
		System.out.println("*  DigerTech Inc. ICL File Writer SDK Sample  *");
		System.out.println("*                                             *");
		System.out.println("*               Version : " + FPLicenseInfo.VERSION + "               *");
		System.out.println("***********************************************");
		System.out.println("                                              ");

		// Make sure the license key is set to the SDK before it starts to process files.
		// Set the license key only one time when the JVM starts.
		
		// If the license key is valid, SDK will return the valid product id of the license.
		// othwise the productId is free version
		int productId = DTFileWriter.setLicenseKey(licenseKey);
		
		if (productId == DTFileWriter.SDK_FILE_WRITER_DEVELOPER) {
			System.out.println("This is a Developer version SDK license. SDK can process file less than 500 items.");
		} else if (productId == DTFileWriter.SDK_FILE_WRITER_ENTERPRISE_I) {
			System.out.println("This is an Enterprise I version SDK license. SDK can process file less than 10000 items.");
		} else if (productId == DTFileWriter.SDK_FILE_WRITER_ENTERPRISE_II) {
			System.out.println("This is an Enterprise II version SDK license. SDK can process file unlimited items.");
		} else if (productId == DTFileWriter.SDK_FILE_WRITER_ENTERPRISE_SERVER) {
			System.out.println("This is an Enterprise Server version SDK license. SDK can process file unlimited items.");
 		} else {
			System.out.println("This is a free version SDK license. SDK can process file less than 10 items.");
		}
		
		FPLicenseInfo fpLicenseInfo = DTFileWriter.getLicenseInfo();
		System.out.println("------------------------- ");
		System.out.println("Computer Id             = " + fpLicenseInfo.getComputerId());
		System.out.println("Product Id              = " + fpLicenseInfo.getProductId());
		System.out.println("Version                 = " + fpLicenseInfo.getVersion());
		System.out.println("Permanent               = " + fpLicenseInfo.isPermanent());
		System.out.println("Expired                 = " + fpLicenseInfo.isExpired());
		System.out.println("Maintiance Expired Date = " + fpLicenseInfo.getMaintainanceDate());
		System.out.println("License Code            = " + fpLicenseInfo.getLicenseCode());
		System.out.println("Return Code             = " + fpLicenseInfo.getReturnCode());
		System.out.println("------------------------- ");
	}

	
	public static void main(String args[]) {
		if (args == null || args.length < 1) {
			System.out.println(new StringBuffer().append("Usage:  \n") 
				  .append("java com.digertech.sdk.samples.SampleICLFileWriter [ICL file] [license key]\n")
				  .append("              [ICL file]    - ICL file with full path name.\n")
				  .append("              [license key] - File processing SDK license key.\n")
				  .toString());
		} else {
			// Set the SDK license key before we process any files.
			// If it is better to set SDK license key when the JVM starts.
			if (args.length == 2) {
				setLicenseKey(args[1]);
			} else {
				setLicenseKey();
			}
			
			// Add user records first
			addUserRecord();
			
			File                 file          = new File(args[0]);
			SampleICLFileWriter  writer        = new SampleICLFileWriter();
			
			try {
				// Call the writeRecords method in this sample class
				// to create and write ICL records to the file.
				writer.writeRecords(file, RecordDefines.X937_2003, true);
//				writer.writeRecords(file, RecordDefines.X9100_180, false);
				
			} catch (FileIOException e) {
				System.out.println("Error writing ICL file. " + e.toString());
			}
			
		}
	}

}
